#!/usr/bin/env python3

"""Dumps to standard output all information on a particular eddy,
specified by date and eddy_index, or by node index, or by shape index.

"""

import argparse
import shapefile
from os import path
import pprint
import sys
import util_eddies
import report_graph
import json

parser = argparse.ArgumentParser()
parser.add_argument("directory", help = "containing the three shapefiles")
group = parser.add_mutually_exclusive_group()
group.add_argument("-i", "--ishape", type = int)
group.add_argument("-n", "--node", type = int)
args = parser.parse_args()

handler = util_eddies.open_shpc(args.directory)

if args.ishape:
    ishape = args.ishape
else:
    if args.node:
        with open("node_id_param.json") as f: node_id_param = json.load(f)
        date, eddy_index = report_graph\
            .node_to_date_eddy(args.node, node_id_param["e_overestim"])
    else:
        reply = input("date, eddy_index = ? ").split(",")
        date = int(reply[0])
        eddy_index = int(reply[1])

    # Find ishape:
    if handler["ishape_last"] is None:
        print("ishape_last not found. Iterating through shapes...")

        for ishape, rec \
            in enumerate(handler["readers"]["extremum"].iterRecords()):
            if rec["date"] == date \
               and rec["eddy_index"] == eddy_index:
                break
        else:
            sys.exit("Eddy with this date and index not found")
    else:
        ishape = util_eddies.comp_ishape(handler, date, eddy_index)

print("ishape =", ishape)

for layer in ["extremum", "outermost_contour", "max_speed_contour", "centroid"]:
    if layer in handler["readers"]:
        shapeRec = handler["readers"][layer].shapeRecord(ishape)
        print("\n", layer, ":")

        if shapeRec.shape.shapeType == shapefile.NULL:
            dct = shapeRec.record.as_dict()
            pprint.pprint(dct)
            print("null shape")
        else:
            pprint.pprint(shapeRec.__geo_interface__)
