import shapefile
import math
from os import path
import numpy as np
import sys

def comp_ishape(handler, date, eddy_index):
    """Compute the location in the shapefiles. handler should be a
        dictionary returned by open_shpc.

    """

    k = date - handler["d_init"]
    
    if k == 0:
        ishape = eddy_index - 1
    else:
        ishape = handler["ishape_last"][k - 1] + eddy_index
        
    return ishape

def in_window(point, window):
    llcrnrlon, llcrnrlat, urcrnrlon, urcrnrlat = window
    longit = point[0] + math.ceil((llcrnrlon - point[0]) / 360) * 360
    # (in [llcrnrlon, llcrnrlon + 360[)

    return longit <= urcrnrlon and llcrnrlat <= point[1] <= urcrnrlat

def open_shpc(shpc_dir):
    if not path.isdir(shpc_dir):
        sys.exit(f"open_shpc: {shpc_dir} is not an existing directory")

    handler = {"readers": {}}

    for layer in ["extremum", "outermost_contour", "max_speed_contour",
                  "centroid"]:
        try:
            handler["readers"][layer] \
                = shapefile.Reader(path.join(shpc_dir, layer))
        except shapefile.ShapefileException:
            print(f"Shapefile {layer} not found in {shpc_dir}.")

    handler["d_init"] = handler["readers"]["extremum"].record(0).date
    fname = path.join(shpc_dir, "ishape_last.txt")

    try:
        handler["ishape_last"] = np.loadtxt(fname, dtype = int, ndmin = 1)
    except OSError:
        print("Could not read", fname)
        handler["ishape_last"] = None

    fname = path.join(shpc_dir, "orientation.txt")

    try:
        with open(fname) as f: line = f.readline()
    except FileNotFoundError:
        print("Could not read", fname)
    else:
        handler["cyclones"] = line.strip() == "cyclones"
        
    return handler
