program test_get_1_outerm

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: new_unit, count_lines
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_close, nf95_inq_varid, nf95_get_var, &
       nf95_gw_var
  use jumble, only: deg_to_rad

  use derived_types, only: eddy, shpc, null_ssh_contour, missing_speed
  use get_1_outerm_m, only: get_1_outerm
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use write_eddy_m, only: write_eddy

  implicit none

  integer ncid, varid, unit, n, l, date
  real longitude(2) ! in rad
  real latitude(2) ! in rad
  real step(2) ! in rad
  real, allocatable:: ssh(:, :, :) ! (:, :, 1) sea-surface height, in m

  integer:: ind_targ_extr(2) = [19, 11]
  ! indices in the two dimensions of target extremum

  real:: innermost_level = 0.2933
  ! level of innermost contour, for each extremum
  
  logical:: cyclone = .true.
  TYPE(shpc) hshp
  type(eddy) e

  real, allocatable:: outside_points(:, :) ! (2, :) longitude and
  ! latitude, in rad, of all the significant extrema, except the
  ! target extremum

  namelist /main_nml/ ind_targ_extr, innermost_level, cyclone

  !----------------------------------------------------------------

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  print *, 'Reading from "h.nc"...'
  call nf95_open("h.nc", nf90_nowrite, ncid)

  call nf95_inq_varid(ncid, "lon", varid)
  call nf95_get_var(ncid, varid, longitude)
  longitude = longitude * deg_to_rad

  call nf95_inq_varid(ncid, "lat", varid)
  call nf95_get_var(ncid, varid, latitude)
  latitude = latitude * deg_to_rad

  call nf95_inq_varid(ncid, "adt", varid)
  call nf95_gw_var(ncid, varid, ssh)

  call nf95_close(ncid)
  print *, "date = ?"
  read *, date
  step = [longitude(2) - longitude(1), latitude(2) - latitude(1)]

  print *, "Reading from outside_points.csv..."
  call new_unit(unit)
  open(unit, file = "outside_points.csv", status = "old", action = "read", &
       position = "rewind")
  call count_lines(unit, n)
  n = n - 2 ! 2 title lines
  allocate(outside_points(2, n))
  rewind(unit)
  read(unit, fmt = *)
  read(unit, fmt = *)

  do l = 1, n
     read(unit, fmt = *) outside_points(:, l)
  end do

  outside_points = outside_points * deg_to_rad
  close(unit)
  e%coord_extr = [longitude(1), latitude(1)] + (ind_targ_extr - 1) * step

  e%out_cont = get_1_outerm(cyclone, &
       coord_extr = e%coord_extr, &
       innermost_level = innermost_level, outside_points = outside_points, &
       ssh = ssh(:, :, 1), corner = [longitude(1), latitude(1)], step = step)

  if (e%out_cont%closed) then
     e%ssh_extr = ssh(ind_targ_extr(1), ind_targ_extr(2), 1)
     e%cyclone = cyclone
     e%speed_cont = null_ssh_contour()
     e%max_speed = missing_speed
     e%radius4 = - 1
     e%valid = .true.
     call shpc_create(hshp, shpc_dir = "SHPC", cyclone = cyclone)
     call write_eddy(e, hshp, date, i = 1)
     open(unit, file = "SHPC/ishape_last.txt", status = "replace", &
          action = "write")
     write(unit, fmt = *) 0
     CALL shpc_close(hshp)
     print *, 'Created "SHPC".'
  else
     print *, "Could not find an outermost contour."
  end if

end program test_get_1_outerm
